# Generated by roxygen2: do not edit by hand

S3method(compile_model,Problem)
S3method(print,Problem)
S3method(print,Solution)
S3method(print,SolutionSet)
S3method(solve,Problem)
export("%>%")
export(Problem)
export(Solution)
export(SolutionSet)
export(add_actions)
export(add_benefits)
export(add_constraint_area)
export(add_constraint_budget)
export(add_constraint_locked_actions)
export(add_constraint_locked_pu)
export(add_constraint_targets_absolute)
export(add_constraint_targets_relative)
export(add_effects)
export(add_losses)
export(add_objective_max_benefit)
export(add_objective_max_net_profit)
export(add_objective_max_profit)
export(add_objective_min_cost)
export(add_objective_min_fragmentation_action)
export(add_objective_min_fragmentation_pu)
export(add_objective_min_intervention_impact)
export(add_objective_min_loss)
export(add_profit)
export(add_spatial_boundary)
export(add_spatial_distance)
export(add_spatial_knn)
export(add_spatial_queen)
export(add_spatial_relations)
export(add_spatial_rook)
export(compile_model)
export(create_problem)
export(get_actions)
export(get_features)
export(get_pu)
export(get_solution_vector)
export(get_targets)
export(load_sim_features_raster)
export(plot_spatial)
export(plot_spatial_actions)
export(plot_spatial_features)
export(plot_spatial_pu)
export(plot_tradeoff)
export(set_method_augmecon)
export(set_method_epsilon_constraint)
export(set_method_weighted_sum)
export(set_solver)
export(set_solver_cbc)
export(set_solver_cplex)
export(set_solver_gurobi)
export(set_solver_symphony)
export(solve)
exportMethods(create_problem)
importFrom(Rcpp,evalCpp)
importFrom(magrittr,"%>%")
useDynLib(multiscape, .registration = TRUE)
