% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print}
\alias{print}
\alias{print.Problem}
\alias{print.Solution}
\alias{print.SolutionSet}
\title{Print}
\usage{
\method{print}{Problem}(x, ...)

\method{print}{Solution}(x, ...)

\method{print}{SolutionSet}(x, ...)
}
\arguments{
\item{x}{Any object.}

\item{...}{Not used.}
}
\value{
None.
}
\description{
Displays information about an object.
}
\seealso{
\code{\link[base:print]{base::print()}}.
}
\keyword{internal}
